module class_vector

  implicit none
   
  type vector
    real(kind(1.d0)) :: x
    real(kind(1.d0)) :: y
    real(kind(1.d0)) :: z
  end type vector

  interface operator(.cross.)
     module procedure cross_prod
  end interface

  interface operator(+)
    module procedure vec_sum
  end interface

contains

  elemental function cross_prod(a,b)
    type(vector) :: cross_prod
    type(vector), intent(in) :: a, b

    cross_prod%x = a%y * b%z - a%z * b%y
    cross_prod%y = a%z * b%x - a%x * b%z
    cross_prod%z = a%x * b%y - a%y * b%x

  end function cross_prod


  elemental function vec_sum(a,b)
    type(vector) :: vec_sum
    type(vector), intent(in) :: a, b
    
    vec_sum%x = a%x + b%x
    vec_sum%y = a%y + b%y
    vec_sum%z = a%z + b%z

  end function vec_sum

end module class_vector

program prova_vector

  use class_vector

  implicit none

  type(vector), dimension(3) :: v,w,z

  v(1)=vector(1.d0,0.d0,0.d0)
  v(2)=vector(0.d0,1.d0,0.d0)
  v(3)=vector(0.d0,0.d0,1.d0)

  w(1)=v(2)
  w(2)=v(3)
  w(3)=v(1)
  
  z(1)=v(3)
  z(2)=v(1)
  z(3)=v(2)

  write(*,*) v+(w.cross.z)

  end program prova_vector
 
